/*
 * Copyright (c) 2004, University of Illinois,Distributed Systems Group 
 * Large & Dynamic Groups Size Estimation project members:
 * Dimitrios Psaltoulis
 * Dionysios Kostoulas
 * Indranil Gupta
 * Al Demers
 * Ken Birman
 *  All rights reserved.
 *
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * - Redistributions of source code must retain the above copyright
 * notice, this list of conditions and the following disclaimer.
 *
 * - Redistributions in binary form must reproduce the above copyright
 * notice, this list of conditions and the following disclaimer in the
 * documentation and/or other materials provided with the
 * distribution.
 *
 * - Neither the name of the University of Illinois, Urbana-Champaign
 * nor the names of its contributors may be used to endorse or promote
 * products derived from this software without specific prior written
 * permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
 * FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 * COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 */

/*
 * HopGossipInfo.java
 *
 * Created on June 17, 2004, 12:57 PM
 */

package SizeEstimationAlgorithm.HopCounting;

/**
 *
 * @author  dimitrios psaltoulis
 */
public class HopGossipInfo extends GossipInfo{
    HopGossipInfo(int hDist, int msgRcv, int msgSent) {
        super(hDist, msgRcv, msgSent);
        reportTable[0] = 1;
    }
    
    public long estimateSizeAv(int base, int minHopRep) {
        double hopsTotal=0.0;
        double clientReportsTotal=0.0;
        for(int i=0; i<minHopRep; ++i) {
            hopsTotal += reportTable[i] * i;
            clientReportsTotal += reportTable[i];
        }
        for(int i=minHopRep; i<MAX_NUMBER_OF_HOPS; ++i) {
            double clHopNumber = reportTable[i] * Math.pow(base, (i - minHopRep) );
            hopsTotal += clHopNumber * i;
            clientReportsTotal += clHopNumber;
        }
        return Math.round( ( Math.pow(base, (hopsTotal/clientReportsTotal) ) ) );
    }
    
    public long estimateSizeSample(int base, int minHopRep) {
        int sampleSize = (int)Math.pow(base,minHopRep);
        int sum = 0;
        for(int i=0; i<minHopRep; ++i) sum += reportTable[i];
        for(int i=minHopRep; i<MAX_NUMBER_OF_HOPS; ++i) {
            if ( (reportTable[i]>sampleSize) ||
            ( ((i+1)<MAX_NUMBER_OF_HOPS)&&(reportTable[i+1]>=sampleSize) ) )
                reportTable[i] = sampleSize;
            
            sum += (int)(reportTable[i] * Math.pow(base, (i - minHopRep) ) );
        }
        return sum;
    }
    
    public void addReport(int i) { reportTable[i]++;}
    
    private final int MAX_NUMBER_OF_HOPS = 50;
    private int[] reportTable =  new int[MAX_NUMBER_OF_HOPS];
}
