/*
 * Copyright (c) 2004, University of Illinois,Distributed Systems Group 
 * Large & Dynamic Groups Size Estimation project members:
 * Dimitrios Psaltoulis
 * Dionysios Kostoulas
 * Indranil Gupta
 * Al Demers
 * Ken Birman
 *  All rights reserved.
 *
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * - Redistributions of source code must retain the above copyright
 * notice, this list of conditions and the following disclaimer.
 *
 * - Redistributions in binary form must reproduce the above copyright
 * notice, this list of conditions and the following disclaimer in the
 * documentation and/or other materials provided with the
 * distribution.
 *
 * - Neither the name of the University of Illinois, Urbana-Champaign
 * nor the names of its contributors may be used to endorse or promote
 * products derived from this software without specific prior written
 * permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
 * FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 * COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 */

/*
 * Interval.java
 *
 * Created on June 17, 2004, 1:00 PM
 */

package SizeEstimationAlgorithm.IntervalCounting.utils;

import java.io.*;
/**
 *
 * @author  dionysios kostoulas
 */
/* - */
// class of interval of hashID space
// currently hashID space is considered to be divided into 40 size units

public class Interval implements Serializable {
    public Interval(String c, String l, String h) {
        centre = new String(c);
        low = new String(l);
        high = new String(h);
    }
    
    public Interval(int c, int l, int h) {
        centre = new String(unitNumToString(c));
        low = new String(unitNumToString(l));
        high = new String(unitNumToString(h));
    }
    
    private int stringToUnitNum(String aIntervalArg) {
        for (int i = aIntervalArg.length() - 1; i >= 0; i--) {
            if (aIntervalArg.charAt(i) == '0') return (aIntervalArg.length() - 1 - i);
        }
        
        return 40;
    }
    
    private String unitNumToString(int units) {
        
        char[] hv = new char[40];
        int i;
        for (i = 0; i < units; i++) {
            hv[39-i] = 'F';
        }
        
        for (int j = i; j < 40; j++) {
            hv[39-j] = '0';
        }
        String hexValue = new String(hv);
        
        return hexValue;
    }
    
    public String getCentre() { return centre; }
    public String getLow() { return low; }
    public String getHigh() { return high; }
    public int getSize() {
        return (stringToUnitNum(high) - stringToUnitNum(low));
    }
    
    public void setCentre(int units) {
        centre = new String(unitNumToString(units));
    }
    
    public void setLow(int units) {
        low = new String(unitNumToString(units));
    }
    
    public void setHigh(int units) {
        high = new String(unitNumToString(units));
    }
    
    public int changeSize(int step) {
        
        int offset = step / 2;
        
        int newLow = (stringToUnitNum(low) - offset);
        if (newLow >= 0) {
            setLow(newLow);
        }
        else return -1;
        
        int newHigh = (stringToUnitNum(high) + offset);
        if (newHigh <= 40) {
            setHigh(newHigh);
        }
        else return -1;
        
        return getSize();
    }
    
    public int changePositionTo(int position) {
        
        int offset = getSize() / 2;
        
        if (((position - offset) < 0) || ((position + offset) > 40))
            return -1;
        
        else {
            setCentre(position);
            setLow(position - offset);
            setHigh(position + offset);
        }
        
        return position;
    }
    
    private String centre;
    private String low;
    private String high;
}