/*
 * Copyright (c) 2004, University of Illinois,Distributed Systems Group
 * Large & Dynamic Groups Size Estimation project members:
 * Dimitrios Psaltoulis
 * Dionysios Kostoulas
 * Indranil Gupta
 * Al Demers
 * Ken Birman
 *  All rights reserved.
 *
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * - Redistributions of source code must retain the above copyright
 * notice, this list of conditions and the following disclaimer.
 *
 * - Redistributions in binary form must reproduce the above copyright
 * notice, this list of conditions and the following disclaimer in the
 * documentation and/or other materials provided with the
 * distribution.
 *
 * - Neither the name of the University of Illinois, Urbana-Champaign
 * nor the names of its contributors may be used to endorse or promote
 * products derived from this software without specific prior written
 * permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
 * FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 * COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 */

/*
 * ListEntry.java
 *
 * Created on June 17, 2004, 1:08 PM
 */

package SizeEstimationAlgorithm.IntervalCounting.utils;

import java.io.*;
/**
 *
 * @author  dionysios kostoulas
 */
// class of entry for list of peers with hashIDs within interval
public class ListEntry	implements Serializable {
    public ListEntry(HashId id, int hrtbeat, int last) {
        pId = id;
        heartbeat = hrtbeat;
        lastModified = last;
    }
    
    private final int timeout = 100;
    
    public HashId getPid() { return pId; }
    public int getHeartbeat() { return heartbeat; }
    public int getLastModified() { return lastModified; }
    
    public void update(int newHeartbeat, int newLastModified) {
        heartbeat = newHeartbeat;
        lastModified = newLastModified;
    }
    
    public boolean isFresherThan(ListEntry aEntry) {
        if (heartbeat > aEntry.getHeartbeat()) return true;
        else return false;
    }
    
    public boolean isRedundant(int rounds) {
        if ((rounds - lastModified) > timeout) return true;
        else return false;
    }
    
    private HashId pId;
    private int heartbeat;
    private int lastModified;
}
